/*
    This file is part of Funiter,
    Real and complex function iteration software.
    
    alconio.c - conio.c emulation for allegro GFX.

    Copyright (C) 1995-2006 Stijn Wolters.
    Original idea: Ernic Kamerich (University of Nijmegen).
    
    See README for contact information.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    See COPYING for more information.
*/

#include "alconio.h"

TConsole *alconio_init(int cwidth, int cheight, int cstretch, 
    int cfgcol, int cbkcol) 
{
    TConsole  *Console = malloc(sizeof(*Console));

    if(Console == NULL) { return NULL; }

    /* Vertical stretch of font in y-direction */
    
    Console->Stretch = cstretch;
    
    /* Current cursor position inside the virtual screen */
    
    Console->CurX = 0;
    Console->CurY = 0;
    
    /* Set width and height in characters */
    
    Console->CharWidth = cwidth;
    Console->CharHeight = cheight;
    
    /* Determine width and height in pixels instead of chars */
    
    Console->Width = Console->CharWidth << 3;
    Console->Height = text_height(font) * Console->Stretch * 
        Console->CharHeight;
    
    /* Center virtual screen */
    
    Console->XStart = (SCREEN_W - Console->Width) / 2;
    Console->YStart = (SCREEN_H - Console->Height) / 2;
    
    /* Create bitmap for what's currently on screen */
    
    Console->ScreenContents = 
        create_bitmap(Console->Width, Console->Height);
    
    /* Create bitmap for what's under the cursor */
    
    Console->Cursor = create_bitmap(X_C2P(Console, 1), Y_C2P(Console, 1));
    
    /* Create a buffer for the text */
    
    Console->TextBuffer = create_bitmap(SCREEN_W, text_height(font));

    /* Set cursor mode */
    
    Console->CursorMode = _SOLIDCURSOR; 

    /* Set text/background colors */
    
    Console->FgCol = cfgcol; 
    Console->BkCol = cbkcol; 

    /* Store background */
        
    blit(screen, Console->ScreenContents, Console->XStart, Console->YStart, 
        0, 0, Console->Width, Console->Height);
    
    a_clrscr(Console);
               
    return Console;
}

int a_cprintf(TConsole *Console, const char *format, ...)
{
    va_list     valp;
    char        s[120] = "";

    va_start(valp, format);
    uvsprintf(s, format, valp);
    va_end(valp);

    a_cputs(Console, s);

    return 0;
}

int a_cputs(TConsole *Console, const char *s)
{
    textprintf_ex(Console->TextBuffer, font, 0, 0, Console->FgCol, 
        Console->BkCol, "%s", s);
    
    stretch_blit(Console->TextBuffer, screen, 0, 0, gui_strlen(s), 
        text_height(font), 
        X_C2P(Console, Console->CurX) + Console->XStart, 
        Y_C2P(Console, Console->CurY) + Console->YStart, 
        gui_strlen(s), Y_C2P(Console, 1));

    a_gotoxy(Console, Console->CurX, Console->CurY);
    a_gotoxy(Console, Console->CurX + strlen(s), Console->CurY);

    return 0;
}

int a_putch(TConsole *Console, char c)
{
    char        s[2];
    
    snprintf(s, 1, "%c", c);
    a_cputs(Console, s);

    return 0;
}

void a_clrscr(TConsole *Console)
{
    /* Draw a rectangle in the backgroundcolor to erase screen */
    
    set_clip_rect(screen, Console->XStart, Console->YStart, 
        Console->XStart + Console->Width - 1, 
        Console->YStart + Console->Height - 1);
    clear_to_color(screen, Console->BkCol);
    
    /* Set cursor to 0, 0 and store what is under the cursor */

    Console->CurX = 0;
    Console->CurY = 0;
    
    a_gotoxy(Console, 0, 0);

    set_clip_rect(screen, 0, 0, SCREEN_W, SCREEN_H);
  
    return;
}

int a_gotoxy(TConsole *Console, int x, int y)
{
    int         x1, y1, x2, y2, sp;

    if(x >= Console->CharWidth) x = Console->CharWidth - 1; 
    if(y >= Console->CharHeight) y = Console->CharHeight - 1; 
    
    sp = ((x == Console->CurX) && (y == Console->CurY));
    
    x1 = X_C2P(Console, Console->CurX) + Console->XStart;
    y1 = Y_C2P(Console, Console->CurY) + Console->YStart;
    x2 = X_C2P(Console, x) + Console->XStart;
    y2 = Y_C2P(Console, y) + Console->YStart;

    if((Console->CursorMode != _NOCURSOR) && !sp) {

        /* Erase cursor on old position */
        
        blit(Console->Cursor, screen, 0, 0, x1, y1, 
            X_C2P(Console, 1), Y_C2P(Console, 1));
    }
    
    Console->CurX = x;
    Console->CurY = y;
    
    if((Console->CursorMode != _NOCURSOR)) {
    
        /* Draw cursor at new position */

        blit(screen, Console->Cursor, x2, y2, 0, 0,  
            X_C2P(Console, 1), Y_C2P(Console, 1));

        if(!sp) {
            xor_mode(TRUE);
            rectfill(screen, x2, y2, x2 + X_C2P(Console, 1) - 1, 
                y2 + Y_C2P(Console, 1) - 1, makecol(255, 255, 255));
            xor_mode(FALSE);
        }
    }
    
    return 0;
}

int a_wherex(TConsole *Console) 
{ 
    return Console->CurX; 
}

int a_wherey(TConsole *Console) 
{ 
    return Console->CurY; 
}

void _a_setcursortype(TConsole *Console, int mode)
{
    if(mode > 0 && mode < 3) 
        Console->CursorMode = mode;
    return;
}

void a_textcolor(TConsole *Console, int Color) 
{ 
    Console->FgCol = Color; 
    return; 
}

void a_textbackground(TConsole *Console, int Color) 
{ 
    Console->BkCol = Color; 
    return; 
}

int a_getch(void)
{
    return (readkey() & 0xFF);
}

int a_getimage(TConsole *Console, BITMAP *Image)
{
    blit(screen, Image, Console->XStart, Console->YStart, 0, 0, 
        Console->Width, Console->Height);
    
    return 0;
}  

int a_putimage(TConsole *Console, BITMAP *Image)
{
    blit(Image, screen, 0, 0, Console->XStart, Console->YStart, 
        Console->Width, Console->Height);
    
    return 0;
}

void alconio_exit(TConsole *Console)
{
    destroy_bitmap(Console->TextBuffer);
    destroy_bitmap(Console->ScreenContents);
    destroy_bitmap(Console->Cursor);    
    free(Console);

    return;
}
